@@ -1,14 +1,13 @@
 //! ASN.1 `ANY` type.
 
 use crate::{
-    BitString, ByteSlice, Choice, ContextSpecific, Decodable, Decoder, Encodable, Encoder, Error,
-    ErrorKind, GeneralizedTime, Header, Ia5String, Length, Null, OctetString, PrintableString,
-    Result, Sequence, Tag, UtcTime, Utf8String,
+    asn1::*, ByteSlice, Choice, Decodable, Decoder, Encodable, Encoder, Error, ErrorKind, Header,
+    Length, Result, Tag,
 };
 use core::convert::{TryFrom, TryInto};
 
 #[cfg(feature = "oid")]
-use crate::ObjectIdentifier;
+use crate::asn1::ObjectIdentifier;
 
 /// ASN.1 `ANY`: represents any explicitly tagged ASN.1 value.
 ///
@@ -160,24 +159,24 @@ impl<'a> Decodable<'a> for Any<'a> {
         let tag = header.tag;
         let mut value = decoder
             .bytes(header.length)
-            .or_else(|_| decoder.error(ErrorKind::Length { tag }))?;
+            .map_err(|_| decoder.error(ErrorKind::Length { tag }))?;
 
         if has_leading_zero_byte(tag) {
             let (byte, rest) = value
                 .split_first()
                 .ok_or(ErrorKind::Truncated)
-                .or_else(|e| decoder.error(e))?;
+                .map_err(|e| decoder.error(e))?;
 
             // The first octet of a BIT STRING encodes the number of unused bits.
             // We presently constrain this to 0.
             if *byte != 0 {
-                return decoder.error(ErrorKind::Noncanonical);
+                return Err(decoder.error(ErrorKind::Noncanonical { tag }));
             }
 
             value = rest;
         }
 
-        Self::new(tag, value).or_else(|e| decoder.error(e.kind()))
+        Self::new(tag, value).map_err(|e| decoder.error(e.kind()))
     }
 }
 
