@@ -1,7 +1,7 @@
 //! ASN.1 `PrintableString` support.
 
 use crate::{
-    str_slice::StrSlice, Any, Encodable, Encoder, Error, ErrorKind, Length, Result, Tag, Tagged,
+    asn1::Any, str_slice::StrSlice, Encodable, Encoder, Error, Length, Result, Tag, Tagged,
 };
 use core::{convert::TryFrom, fmt, str};
 
@@ -9,8 +9,8 @@ use core::{convert::TryFrom, fmt, str};
 ///
 /// Supports a subset the ASCII character set (desribed below).
 ///
-/// For UTF-8, use [`Utf8String`][`crate::Utf8String`] instead. For the full
-/// ASCII character set, use [`Ia5String`][`crate::Ia5String`].
+/// For UTF-8, use [`Utf8String`][`crate::asn1::Utf8String`] instead. For the
+/// full ASCII character set, use [`Ia5String`][`crate::asn1::Ia5String`].
 ///
 /// # Supported characters
 ///
@@ -63,13 +63,13 @@ impl<'a> PrintableString<'a> {
                 | b':'
                 | b'='
                 | b'?' => (),
-                _ => return Err(ErrorKind::Value { tag: Self::TAG }.into()),
+                _ => return Err(Self::TAG.value_error()),
             }
         }
 
         StrSlice::from_bytes(input)
             .map(|inner| Self { inner })
-            .map_err(|_| ErrorKind::Value { tag: Self::TAG }.into())
+            .map_err(|_| Self::TAG.value_error())
     }
 
     /// Borrow the string as a `str`.
