@@ -1,19 +1,10 @@
 //! DER decoder.
 
-use crate::{
-    Any, BitString, Choice, ContextSpecific, Decodable, ErrorKind, GeneralizedTime, Ia5String,
-    Length, Null, OctetString, PrintableString, Result, Sequence, UtcTime, Utf8String,
+use crate::{asn1::*, Choice, Decodable, Error, ErrorKind, Length, Result, Tag, TagNumber};
+use core::{
+    cmp::Ordering,
+    convert::{TryFrom, TryInto},
 };
-use core::convert::{TryFrom, TryInto};
-
-#[cfg(feature = "big-uint")]
-use {
-    crate::BigUInt,
-    typenum::{NonZero, Unsigned},
-};
-
-#[cfg(feature = "oid")]
-use crate::ObjectIdentifier;
 
 /// DER decoder.
 #[derive(Debug)]
@@ -40,7 +31,7 @@ impl<'a> Decoder<'a> {
     /// Decode a value which impls the [`Decodable`] trait.
     pub fn decode<T: Decodable<'a>>(&mut self) -> Result<T> {
         if self.is_failed() {
-            self.error(ErrorKind::Failed)?;
+            return Err(self.error(ErrorKind::Failed));
         }
 
         T::decode(self).map_err(|e| {
@@ -51,9 +42,14 @@ impl<'a> Decoder<'a> {
 
     /// Return an error with the given [`ErrorKind`], annotating it with
     /// context about where the error occurred.
-    pub fn error<T>(&mut self, kind: ErrorKind) -> Result<T> {
+    pub fn error(&mut self, kind: ErrorKind) -> Error {
         self.bytes.take();
-        Err(kind.at(self.position))
+        kind.at(self.position)
+    }
+
+    /// Return an error for an invalid value with the given tag.
+    pub fn value_error(&mut self, tag: Tag) -> Error {
+        self.error(tag.value_error().kind())
     }
 
     /// Did the decoding operation fail due to an error?
@@ -115,13 +111,10 @@ impl<'a> Decoder<'a> {
         self.decode()
     }
 
-    /// Attempt to decode an ASN.1 `INTEGER` as a [`BigUInt`].
-    #[cfg(feature = "big-uint")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "big-uint")))]
-    pub fn big_uint<N>(&mut self) -> Result<BigUInt<'a, N>>
-    where
-        N: Unsigned + NonZero,
-    {
+    /// Attempt to decode an ASN.1 `INTEGER` as a [`UIntBytes`].
+    #[cfg(feature = "bigint")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "bigint")))]
+    pub fn uint_bytes(&mut self) -> Result<UIntBytes<'a>> {
         self.decode()
     }
 
@@ -130,14 +123,42 @@ impl<'a> Decoder<'a> {
         self.decode()
     }
 
-    /// Attempt to decode an ASN.1 `CONTEXT-SPECIFIC` field.
-    pub fn context_specific(&mut self) -> Result<ContextSpecific<'a>> {
-        self.decode()
-    }
-
-    /// Attempt to decode an `OPTIONAL` ASN.1 `CONTEXT-SPECIFIC` field.
-    pub fn context_specific_optional(&mut self) -> Result<Option<ContextSpecific<'a>>> {
-        self.decode()
+    /// Attempt to decode an ASN.1 `CONTEXT-SPECIFIC` field with the
+    /// provided [`TagNumber`].
+    ///
+    /// This method has the following behavior which is designed to simplify
+    /// handling of extension fields, which are denoted in an ASN.1 schema
+    /// using the `...` ellipsis extension marker:
+    ///
+    /// - Skips over [`ContextSpecific`] fields with a tag number lower than
+    ///   the current one, consuming and ignoring them.
+    /// - Returns `Ok(None)` if a [`ContextSpecific`] field with a higher tag
+    ///   number is encountered. These fields are not consumed in this case,
+    ///   allowing a field with a lower tag number to be omitted, then the
+    ///   higher numbered field consumed as a follow-up.
+    /// - Returns `Ok(None)` if anything other than a [`ContextSpecific`] field
+    ///   is encountered.
+    pub fn context_specific(&mut self, tag: TagNumber) -> Result<Option<Any<'a>>> {
+        loop {
+            match self.peek().map(Tag::try_from).transpose()? {
+                Some(Tag::ContextSpecific(actual_tag)) => {
+                    match actual_tag.cmp(&tag) {
+                        Ordering::Less => {
+                            // Decode and ignore lower-numbered fields if
+                            // they parse correctly.
+                            self.decode::<ContextSpecific<'_>>()?;
+                        }
+                        Ordering::Equal => {
+                            return self
+                                .decode::<ContextSpecific<'_>>()
+                                .map(|cs| Some(cs.value))
+                        }
+                        Ordering::Greater => return Ok(None),
+                    }
+                }
+                _ => return Ok(None),
+            }
+        }
     }
 
     /// Attempt to decode an ASN.1 `GeneralizedTime`.
@@ -203,7 +224,7 @@ impl<'a> Decoder<'a> {
     pub(crate) fn byte(&mut self) -> Result<u8> {
         match self.bytes(1u8)? {
             [byte] => Ok(*byte),
-            _ => self.error(ErrorKind::Truncated),
+            _ => Err(self.error(ErrorKind::Truncated)),
         }
     }
 
@@ -211,12 +232,12 @@ impl<'a> Decoder<'a> {
     /// position, or return an error if we have insufficient data.
     pub(crate) fn bytes(&mut self, len: impl TryInto<Length>) -> Result<&'a [u8]> {
         if self.is_failed() {
-            self.error(ErrorKind::Failed)?;
+            return Err(self.error(ErrorKind::Failed));
         }
 
         let len = len
             .try_into()
-            .or_else(|_| self.error(ErrorKind::Overflow))?;
+            .map_err(|_| self.error(ErrorKind::Overflow))?;
 
         let result = self
             .remaining()?
@@ -259,7 +280,42 @@ impl<'a> From<&'a [u8]> for Decoder<'a> {
 #[cfg(test)]
 mod tests {
     use super::Decoder;
-    use crate::{Decodable, ErrorKind, Length, Tag};
+    use crate::{Decodable, ErrorKind, Length, Tag, TagNumber};
+    use core::convert::TryFrom;
+    use hex_literal::hex;
+
+    #[test]
+    fn context_specific_with_expected_field() {
+        let tag = TagNumber::new(0);
+
+        // Empty message
+        let mut decoder = Decoder::new(&[]);
+        assert_eq!(decoder.context_specific(tag).unwrap(), None);
+
+        // Message containing a non-context-specific type
+        let mut decoder = Decoder::new(&hex!("020100"));
+        assert_eq!(decoder.context_specific(tag).unwrap(), None);
+
+        //
+        let mut decoder = Decoder::new(&hex!("A003020100"));
+        let field = decoder.context_specific(tag).unwrap().unwrap();
+        assert_eq!(u8::try_from(field).unwrap(), 0);
+    }
+
+    #[test]
+    fn context_specific_skipping_unknown_field() {
+        let tag = TagNumber::new(1);
+        let mut decoder = Decoder::new(&hex!("A003020100A103020101"));
+        let field = decoder.context_specific(tag).unwrap().unwrap();
+        assert_eq!(u8::try_from(field).unwrap(), 1);
+    }
+
+    #[test]
+    fn context_specific_returns_none_on_greater_tag_number() {
+        let tag = TagNumber::new(0);
+        let mut decoder = Decoder::new(&hex!("A103020101"));
+        assert_eq!(decoder.context_specific(tag).unwrap(), None);
+    }
 
     #[test]
     fn truncated_message() {
